"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.exportTestkits = exports.warningBanner = void 0;
const tslib_1 = require("tslib");
const path_1 = tslib_1.__importDefault(require("path"));
const fs_1 = tslib_1.__importDefault(require("fs"));
const ejs_1 = tslib_1.__importDefault(require("ejs"));
const camelCase_1 = tslib_1.__importDefault(require("lodash/camelCase"));
const kebabCase_1 = tslib_1.__importDefault(require("lodash/kebabCase"));
const snakeCase_1 = tslib_1.__importDefault(require("lodash/snakeCase"));
const file_exists_1 = require("../../file-exists");
const object_entries_1 = require("../../object-entries");
const pathResolver = (cwd) => (...a) => path_1.default.resolve(cwd, ...a);
const warningBanner = (templatePath) => `/* eslint-disable */
/* tslint:disable */
/*
 * DO NOT EDIT THIS FILE!
 * YOUR CHANGES WILL BE OVERWRITTEN!
 * FILE IS BASED ON ${templatePath}
 * AND GENERATED BY \`wuf export-testkits\`
 */`;
exports.warningBanner = warningBanner;
const writeFile = (pathname, source) => {
    try {
        fs_1.default.writeFileSync(pathname, source);
    }
    catch (e) {
        throw new Error(`Unable to generate testkits: ${e}`);
    }
};
const resolveDefinitions = ({ requirePath, cwd, components }) => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
    try {
        const definitions = require(requirePath);
        if (typeof definitions === 'function') {
            return yield definitions({ components, cwd });
        }
        return definitions;
    }
    catch (e) {
        const relativePath = path_1.default.relative(cwd, path_1.default.resolve(requirePath));
        throw new Error(`Unable to load definitions file at "${relativePath}":\n${e}`);
    }
});
const resolveComponents = ({ requirePath, cwd }) => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
    if (!(yield (0, file_exists_1.fileExists)(requirePath))) {
        throw new Error(`Components file not found at "${requirePath}". It is required for \`wuf export-testkits\`. Create one with \`wuf update\`.`);
    }
    try {
        return require(path_1.default.resolve(cwd, requirePath));
    }
    catch (e) {
        const relativePath = path_1.default.relative(cwd, path_1.default.resolve(requirePath));
        throw new Error(`Unable to load ${requirePath} file at "${relativePath}":\n${e}`);
    }
});
const resolveTemplate = ({ requirePath }) => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
    if (!(yield (0, file_exists_1.fileExists)(requirePath))) {
        throw new Error(`Template file not found at "${requirePath}". It is required for \`wuf export-testkits\`.`);
    }
    try {
        return fs_1.default.readFileSync(requirePath, 'utf8');
    }
    catch (e) {
        //
    }
});
const guards = (optionsRaw) => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
    const pathResolve = pathResolver(optionsRaw._process.cwd);
    if (!optionsRaw.output) {
        throw new Error('missing --output parameter, it must be defined');
    }
    const components = yield resolveComponents({
        requirePath: pathResolve(optionsRaw.components || '.wuf/components.json'),
        cwd: optionsRaw._process.cwd,
    });
    const definitions = optionsRaw.definitions
        ? yield resolveDefinitions({
            requirePath: pathResolve(optionsRaw.definitions),
            cwd: optionsRaw._process.cwd,
            components,
        })
        : {};
    const templatePath = pathResolve(optionsRaw.template || '.wuf/testkits/template.ejs');
    const template = yield resolveTemplate({
        requirePath: templatePath,
    });
    const options = Object.assign(Object.assign({}, optionsRaw), { definitions,
        components,
        template,
        templatePath, output: pathResolve(optionsRaw.output), 
        /** @deprecated it should be set in template */
        factoryName: optionsRaw.factoryName || 'testkitFactoryCreator', 
        /** @deprecated it should be set in template */
        uniFactoryName: optionsRaw.uniFactoryName || 'uniTestkitFactoryCreator', 
        /** @deprecated it should be set in template */
        exportSuffix: optionsRaw.exportSuffix || 'TestkitFactory', 
        /** @deprecated it should be set in template */
        exportCaseStyle: optionsRaw.exportCaseStyle || 'camelCase' });
    return makeOutput(options);
});
const ejsSource = ({ source, definitions, components, data }) => {
    const utils = {
        toCamel: camelCase_1.default,
        toKebab: kebabCase_1.default,
        toSnake: snakeCase_1.default,
        toPascal: (s) => {
            const camel = (0, camelCase_1.default)(s);
            return camel[0].toUpperCase() + camel.substring(1);
        },
    };
    const componentsForEjs = (0, object_entries_1.objectEntries)(Object.assign(Object.assign({}, components), definitions)).map(([name, value]) => (Object.assign(Object.assign({ name }, value), (definitions[name] || {}))));
    try {
        return ejs_1.default.render(source, {
            utils,
            components: componentsForEjs,
            data,
        });
    }
    catch (e) {
        throw new Error(`Erroneous template file: ${e}`);
    }
};
const makeOutput = (options) => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
    const testkitsExportsSource = ejsSource({
        source: options.template,
        definitions: options.definitions,
        data: options.definitions,
        components: options.components,
    });
    const source = [
        (0, exports.warningBanner)(path_1.default.relative(options._process.cwd, path_1.default.resolve(options.templatePath))),
        testkitsExportsSource,
    ].join('\n');
    writeFile(options.output, source);
});
exports.exportTestkits = guards;
//# sourceMappingURL=index.js.map