"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.copyTemplates = void 0;
const tslib_1 = require("tslib");
const fs_1 = tslib_1.__importDefault(require("fs"));
const path_1 = tslib_1.__importDefault(require("path"));
const mkdirp_1 = tslib_1.__importDefault(require("mkdirp"));
const kebabCase_1 = tslib_1.__importDefault(require("lodash/kebabCase"));
const camelCase_1 = tslib_1.__importDefault(require("lodash/camelCase"));
const snakeCase_1 = tslib_1.__importDefault(require("lodash/snakeCase"));
const replace_templates_1 = require("./replace-templates");
const create_values_map_1 = require("../create-values-map");
const interpolate_file_name_1 = require("../interpolate-file-name");
const pathExists = (p) => new Promise((resolve) => {
    fs_1.default.access(p, fs_1.default.constants.F_OK, (err) => {
        resolve(!err);
    });
});
const copyTemplates = ({ templates, ComponentName, description, _process, output = '', }) => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
    if (!ComponentName) {
        throw new Error('Component name must be provided!');
    }
    const readdir = (entry) => fs_1.default.readdirSync(entry).map((dir) => path_1.default.join(entry, dir));
    const componentNames = {
        ComponentName,
        componentName: (0, camelCase_1.default)(ComponentName),
        'component-name': (0, kebabCase_1.default)(ComponentName),
        component_name: (0, snakeCase_1.default)(ComponentName),
    };
    const queue = readdir(templates);
    while (queue.length) {
        const itemPath = queue.shift();
        const isDir = fs_1.default.lstatSync(itemPath).isDirectory();
        const renamedDestPath = (0, interpolate_file_name_1.interpolateFileName)({
            filename: path_1.default.join(_process.cwd, output, itemPath.replace(templates, '')),
            names: componentNames,
        });
        if (isDir) {
            queue.push(...readdir(path_1.default.join(itemPath)));
            if (!(yield pathExists(renamedDestPath))) {
                mkdirp_1.default.sync(renamedDestPath);
            }
        }
        else if (!(yield pathExists(renamedDestPath))) {
            const replaced = (0, replace_templates_1.replaceTemplates)(fs_1.default.readFileSync(itemPath, 'utf8'), (0, create_values_map_1.createValuesMap)({ ComponentName, description }));
            fs_1.default.writeFileSync(renamedDestPath, replaced, 'utf8');
        }
    }
});
exports.copyTemplates = copyTemplates;
//# sourceMappingURL=copy-templates.js.map