"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.replaceTemplates = void 0;
const tslib_1 = require("tslib");
const ejs_1 = tslib_1.__importDefault(require("ejs"));
const camelCase_1 = tslib_1.__importDefault(require("lodash/camelCase"));
const kebabCase_1 = tslib_1.__importDefault(require("lodash/kebabCase"));
const snakeCase_1 = tslib_1.__importDefault(require("lodash/snakeCase"));
const utils = {
    toCamel: camelCase_1.default,
    toKebab: kebabCase_1.default,
    toSnake: snakeCase_1.default,
    toPascal: (s) => {
        const camel = (0, camelCase_1.default)(s);
        return camel[0].toUpperCase() + camel.substring(1);
    },
};
const replaceTemplates = (source, scope) => {
    /*
     * TODO: usage of `{%` & `%}` tags is deprecated!
     * regular EJS should be used in user templates
     *
     * keeping it to not break existing users
     * can be removed either:
     * 1. with major version release
     * 2. when ensured no consumers use it
     */
    const replaceDeprecatedSyntax = source.replace(/{%[\w-]+%}/g, (match) => {
        const key = match.slice(2, -2);
        if (!(key in scope)) {
            throw new Error(`key '${key}' must be one of: [${Object.keys(scope)}]`);
        }
        return scope[key];
    });
    try {
        return ejs_1.default.render(replaceDeprecatedSyntax, Object.assign({ utils }, scope));
    }
    catch (e) {
        throw new Error(`Erroneous template syntax: ${e}`);
    }
};
exports.replaceTemplates = replaceTemplates;
//# sourceMappingURL=replace-templates.js.map