"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.runCodemods = void 0;
const tslib_1 = require("tslib");
const fs_1 = tslib_1.__importDefault(require("fs"));
const path_1 = tslib_1.__importDefault(require("path"));
const child_process_1 = require("child_process");
const logger = tslib_1.__importStar(require("../../../logger"));
const create_values_map_1 = require("../create-values-map");
const runCodemod = ({ options, codemodValues, codemodConfig }) => new Promise((resolve, reject) => {
    const codemodPath = path_1.default.join(options.codemods, codemodConfig.codemod);
    const pathToExecutable = path_1.default.join(options._process.cwd, 'node_modules', '.bin', 'jscodeshift');
    const inputPath = path_1.default.join(options._process.cwd, codemodConfig.dist);
    if (!fs_1.default.existsSync(inputPath)) {
        reject(`Error in ${codemodConfig.codemod}: ${inputPath} does not exist`);
        return;
    }
    const command = `${pathToExecutable} \
      ${inputPath} \
      -t ${codemodPath} \
      --ComponentName=${codemodValues.ComponentName} \
      --componentName=${codemodValues.componentName} \
      --component-name=${codemodValues['component-name']} \
      --verbose=2`;
    const execProc = (0, child_process_1.exec)(command);
    execProc.stderr.on('data', (data) => {
        logger.error(`Error in ${codemodConfig.codemod}: ${data.toString()}`);
        reject(data.toString());
    });
    execProc.on('exit', () => {
        logger.success(codemodConfig.description);
        resolve();
    });
});
const runCodemods = (options) => {
    const { ComponentName, description, codemods } = options;
    const codemodsIndex = require(path_1.default.resolve(options._process.cwd, codemods));
    const codemodValues = (0, create_values_map_1.createValuesMap)({
        ComponentName,
        description,
    });
    return codemodsIndex.reduce((promise, codemodConfig) => promise.then(() => runCodemod({
        options,
        codemodValues,
        codemodConfig,
    })), Promise.resolve());
};
exports.runCodemods = runCodemods;
//# sourceMappingURL=run-codemods.js.map