"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.make = void 0;
const tslib_1 = require("tslib");
const path_1 = tslib_1.__importDefault(require("path"));
const object_entries_1 = require("../../object-entries");
const errors_1 = require("../../errors");
const template_1 = require("./template");
const plugin_1 = require("./plugin");
const path_resolver_1 = require("./path-resolver");
const resolveInput = ({ requirePath, cwd }) => {
    try {
        const inputPath = (0, errors_1.resolveOrThrow)(() => require.resolve(path_1.default.resolve(cwd, requirePath)), (error) => new errors_1.WufError({
            name: 'RequireResolveError',
            kind: "SystemError" /* SystemError */,
            message: `Unable to find file at ${requirePath}`,
            error,
            info: {
                requirePath,
            },
        }));
        const input = (0, errors_1.resolveOrThrow)(() => require(inputPath), (error) => new errors_1.WufError({
            name: `RequireError`,
            kind: "SystemError" /* SystemError */,
            message: 'Unable to require file provided in `--input`',
            error,
        }));
        return input;
    }
    catch (error) {
        if (error instanceof errors_1.WufError &&
            error.name === 'RequireResolveError' &&
            error.info.requirePath.endsWith('.wuf/components.json')) {
            // This is a recoverable error:
            // 1. User did not provide `--input`
            // 2. fallback `.wuf/components.json` was not found
            // program should not fail, empty object is valid value
            return {};
        }
        else {
            throw error;
        }
    }
};
const makeOutput = (options) => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
    const dataFromPlugins = yield (0, plugin_1.resolvePlugin)({
        options,
        components: (0, object_entries_1.objectEntries)(options.components).map(([name, value]) => (Object.assign({ name }, value))),
    });
    if (options.template) {
        yield (0, template_1.renderTemplate)({
            templatePath: options.template,
            output: options.output,
            data: dataFromPlugins,
        });
    }
});
const processRawOptions = (optionsRaw) => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
    if (!optionsRaw.output) {
        throw new errors_1.WufError({
            name: 'OptionsError',
            kind: "UserError" /* UserError */,
            message: 'Missing required `output` parameter.',
        });
    }
    const pathResolve = (0, path_resolver_1.pathResolver)(optionsRaw._process.cwd);
    const options = Object.assign(Object.assign({}, optionsRaw), { plugin: optionsRaw.plugin || [], template: optionsRaw.template ? pathResolve(optionsRaw.template) : null, output: pathResolve(optionsRaw.output), components: resolveInput({
            requirePath: pathResolve(optionsRaw.input || '.wuf/components.json'),
            cwd: optionsRaw._process.cwd,
        }) });
    return makeOutput(options);
});
exports.make = processRawOptions;
//# sourceMappingURL=index.js.map