"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.renderTemplate = void 0;
const tslib_1 = require("tslib");
const fs_extra_1 = tslib_1.__importDefault(require("fs-extra"));
const ejs_1 = tslib_1.__importDefault(require("ejs"));
const file_exists_1 = require("../../file-exists");
const errors_1 = require("../../errors");
const string_utils_1 = require("./string-utils");
const resolveTemplate = ({ requirePath }) => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
    if (!(yield (0, file_exists_1.fileExists)(requirePath))) {
        throw new errors_1.WufError({
            kind: "UserError" /* UserError */,
            name: 'TemplateError',
            message: `file not found at "${requirePath}".`,
        });
    }
    try {
        return yield fs_extra_1.default.readFile(requirePath, 'utf8');
    }
    catch (error) {
        throw new errors_1.WufError({
            kind: "SystemError" /* SystemError */,
            name: 'TemplateError',
            message: `Unable to read template file at "${requirePath}"`,
            error,
        });
    }
});
const renderTemplate = ({ templatePath, output, data, }) => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
    const template = yield resolveTemplate({ requirePath: templatePath });
    if (template) {
        const source = ejsRender({ template, data });
        yield fs_extra_1.default.outputFile(output, source, { encoding: 'utf8' });
    }
});
exports.renderTemplate = renderTemplate;
const ejsRender = ({ template, data }) => {
    try {
        return ejs_1.default.render(template, Object.assign(Object.assign({}, data), { utils: string_utils_1.stringUtils }));
    }
    catch (error) {
        throw new errors_1.WufError({
            name: 'EjsError',
            kind: "SystemError" /* SystemError */,
            message: `Template file contains error`,
            error,
        });
    }
};
//# sourceMappingURL=template.js.map