"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getDirtyComponents = void 0;
const tslib_1 = require("tslib");
const path_1 = tslib_1.__importDefault(require("path"));
const dependency_tree_1 = tslib_1.__importDefault(require("dependency-tree"));
const resolve_require_1 = require("../resolve-require");
const getDirtyComponents = ({ rootPath, components, changedFiles: changedFilesRaw, }) => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
    const changedFiles = changedFilesRaw.map((changedFilePath) => path_1.default.resolve(changedFilePath));
    const visited = {};
    const getDependencies = (componentPath) => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
        const filename = yield (0, resolve_require_1.resolveRequire)(path_1.default.resolve(rootPath, componentPath));
        return dependency_tree_1.default.toList({
            filename,
            directory: path_1.default.dirname(filename),
            visited,
            filter: (p) => [
                ['.js', '.jsx', '.ts', '.tsx', '.st.css', '.scss'].some((extension) => p.endsWith(extension)),
                !p.includes('node_modules'),
                p !== path_1.default.resolve(rootPath, 'src/index.js'),
            ].every(Boolean),
        });
    });
    const componentsWithDependencyList = yield Object.entries(components).reduce((accPromise, current) => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
        const acc = yield accPromise;
        const [componentName, { path: componentPath }] = current;
        acc[componentName] = yield getDependencies(componentPath);
        return acc;
    }), Promise.resolve({}));
    const dirtyComponents = Object.entries(componentsWithDependencyList).reduce((acc, [componentName, dependencies]) => {
        const isDirty = dependencies.some((dependency) => {
            return (changedFiles.includes(dependency) ||
                changedFiles.some((changedFile) => {
                    return changedFile.includes(path_1.default.resolve(components[componentName].path));
                }));
        });
        if (isDirty) {
            acc.add(componentName);
        }
        return acc;
    }, new Set());
    return Array.from(dirtyComponents);
});
exports.getDirtyComponents = getDirtyComponents;
//# sourceMappingURL=index.js.map