"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.jsonToFs = void 0;
const tslib_1 = require("tslib");
const promises_1 = tslib_1.__importDefault(require("fs/promises"));
const path_1 = tslib_1.__importDefault(require("path"));
const jsonToFs = ({ tree = {}, cwd }) => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
    if (!cwd) {
        return Promise.reject('jsonToFs expects `cwd` to be passed');
    }
    const makeQueue = (tree) => Object.entries(tree).map((entry) => ({
        type: 'files',
        entry,
    }));
    let currentPath = cwd;
    const queue = makeQueue(tree);
    while (queue.length) {
        const current = queue.shift();
        if (current.type === 'files') {
            const [name, content] = current.entry;
            if (typeof content === 'string') {
                yield promises_1.default.writeFile(path_1.default.join(currentPath, name), content, {
                    encoding: 'utf8',
                });
            }
            else {
                queue.unshift({ type: 'down', name }, ...makeQueue(content), {
                    type: 'up',
                });
            }
        }
        if (current.type === 'down') {
            currentPath = path_1.default.join(currentPath, current.name);
            try {
                yield promises_1.default.mkdir(currentPath);
            }
            catch (e) {
                // TODO: consider a nicer handling for the case when we try to create folder that already exists
            }
        }
        if (current.type === 'up') {
            currentPath = path_1.default.dirname(currentPath);
        }
    }
});
exports.jsonToFs = jsonToFs;
//# sourceMappingURL=index.js.map