"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.resolveRequire = void 0;
const tslib_1 = require("tslib");
const path_1 = tslib_1.__importDefault(require("path"));
const errors_1 = require("../errors");
const supportedExtensions = ['.js', '.jsx', '.ts', '.tsx'];
const makeResolvableFiles = (modulePath) => {
    const basename = path_1.default.basename(modulePath);
    return [
        ...supportedExtensions.map((ext) => `index${ext}`),
        ...supportedExtensions.map((ext) => `${basename}${ext}`),
    ];
};
const requireResolve = (p) => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
    return new Promise((resolve, reject) => {
        try {
            resolve(require.resolve(p));
        }
        catch (e) {
            reject(e);
        }
    });
});
const invert = (promise) => new Promise((resolve, reject) => promise.then(reject).catch(resolve));
const first = (promises) => invert(Promise.all(promises.map(invert)));
const resolveRequire = (modulePath) => tslib_1.__awaiter(void 0, void 0, void 0, function* () {
    if (!modulePath) {
        throw new errors_1.WufError({
            name: 'ResolveRequire',
            kind: "UserError" /* UserError */,
            message: 'missing required path argument',
        });
    }
    try {
        return require.resolve(modulePath);
    }
    catch (e) {
        return first(makeResolvableFiles(modulePath).map((p) => requireResolve(path_1.default.join(modulePath, p))));
    }
});
exports.resolveRequire = resolveRequire;
//# sourceMappingURL=index.js.map